from typing import Type, Union

from selenium import webdriver
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.by import By

from afs_autotests.afs_base_objects.base_structure import BasePageStructure
from afs_autotests.afs_base_objects.base_waiting_elements import WaitingElement, WaitingElementCollection
from afs_autotests.exceptions import DriverNotImplementedException, AFSConfirmPageException, PageHrefNotImplemented
from afs_autotests.objects import element_conditions


class BasePageObject(object):
    """

    Базовая страница.
    Attributes:
        driver: Selenium Web Driver object
        href: Ссылка страницы.
        project_main_href: Ссылка на главную страницу проекта.
        timeout: Таймаут на открытие страницы.
        confirm: Требуется ли поиск элемента подтверждения страницы.
        init: Требуется ли открыть страницу
        structure: Структура страницы
    """

    PAGE_NAME = "BasePage"

    def __init__(self, driver: Union[webdriver.Firefox, webdriver.Chrome], project_main_href, timeout=10, href=None, confirm=True, init=False, structure=BasePageStructure):
        """Init BasePageObject"""

        if not driver:
            raise DriverNotImplementedException('Driver not implemented.')

        self.project_main_href = project_main_href
        self.driver = driver
        self.href = href
        self.timeout = timeout
        self.__structure = structure

        if init:
            if self.href:
                self.open_self_page()
            else:
                raise PageHrefNotImplemented("Page href not implemented")

        if confirm:
            self.confirm()

    def search(self, selector, selector_type=By.XPATH, element_type=WaitingElement, timeout=None) -> WaitingElement:
        """Поиск элемента на странице."""
        timeout = timeout if timeout else self.timeout
        return element_type(self.driver, selector, selector_type, timeout=timeout)

    def search_all(self, selector, selector_type=By.XPATH, elements_type=WaitingElement) -> WaitingElementCollection:
        """Поиск всех элементов на странице удовлетворяющим xpath."""
        return WaitingElementCollection(self.driver, selector, selector_type, elements_type)

    def confirm(self):
        """Подтверждение что открылась нужная страница."""
        try:
            self.search(self.__structure.CONFIRM).assure(element_conditions.exist, timeout=self.timeout)
        except TimeoutException as e:
            raise AFSConfirmPageException(
                "Can`t open \"{0}\" for {1} sec.".format(self.PAGE_NAME, self.timeout))

    def open_self_page(self):
        """Открыть страницу в браузере."""
        self.driver.get(self.href)

    def get_alert(self) -> Alert:
        """Получить элемент Alert если он присутствует на странице"""
        return self.driver.switch_to.alert

    def back(self, page_class, timeout=None, **kwargs):
        """Вернуть браузер на предыдущую страницу"""
        self.driver.back()
        timeout = self.timeout if timeout is None else timeout
        return page_class(self.driver, self.href, timeout, **kwargs)

    @property
    def structure(self) -> Type[BasePageStructure]:
        """Структура страницы"""
        return self.__structure
