from typing import Optional, Union

from selenium.webdriver import Firefox, Chrome
from selenium.common.exceptions import TimeoutException, NoSuchElementException
from selenium.webdriver import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait


class WaitingElementCollection(object):
    """

    Коллектция WaitingElement. Ищет все элементы с указанным xpath.

    Attributes:
        driver: WebDriver
        selector: Селектор элементов
        selector_type: Тип селектора
        elements_type: Тип элементов, по умолчанию WaitingElement
        timeout: Время на поиск элемента
        kwargs: Дополнительные параметры
    """

    def __init__(self, driver: Union[Firefox, Chrome], selector: str, selector_type: By, elements_type=None,
                 timeout: int = 20, **kwargs):

        self._driver = driver
        self._selector_type = selector_type

        self._elements_type = elements_type if elements_type is not None else WaitingElement
        self._selector = selector
        self._timeout = timeout

        self._selectors = None

        self._generate_selectors()
        self.current = 0
        self.kwargs = kwargs

    def _generate_selectors(self):
        """Генерирует селекторы элементов"""
        def generate_selector(selector, index):
            return "({0})[{1}]".format(selector, str(index))
        self._selectors = [generate_selector(self._selector, item + 1) for item in range(0, self._get_collection_len())]

    def _get_collection_len(self) -> int:
        """Количество элементов коллекции"""
        return len(self._driver.find_elements(by=self._selector_type, value=self._selector))

    def _create_element(self, index: int) -> WaitingElement:
        """Создаем элемент"""
        return self._elements_type(self._driver,
                                   self._selectors[index],
                                   self._selector_type,
                                   self._timeout,
                                   is_collection_item=True,
                                   **self.kwargs)

    def __getitem__(self, index):
        """

        :type index: int
        """
        self._generate_selectors()
        if isinstance(index, slice):
            stop = self.__len__() if index.stop is None else index.stop
            return [self._create_element(i) for i in range(index.start, stop)]
        return self._create_element(index)

    def __len__(self) -> int:
        self._generate_selectors()
        return len(self._selectors)

    def __iter__(self):
        """

        :rtype: WaitingElementCollection
        """
        self.current = 0
        return self

    def __next__(self):
        if self.current == self.__len__():
            raise StopIteration
        s = self[self.current]
        self.current += 1
        return s

    def assure(self, condition, *args, **kwargs):
        """Возвращает коллекцию если выполняется условие condition"""
        self._generate_selectors()
        timeout = self._timeout if 'timeout' not in kwargs.keys() else kwargs['timeout']
        condition = condition(self, *args)
        WebDriverWait(self._driver, timeout).until(condition, str(self) + condition.error_msg_postfix)
        return self


class BaseWaitingElement(object):
    """

    BaseWaitingElement. Ищет элемент с указанным xpath.

    Attributes:
        driver: WebDriver
        selector: Селектор элементов
        selector_type: Тип селектора
        timeout: Время на поиск элемента
        is_collection_item: Является ли элементов коллекции. Используется при обьявлении WaitingElementCollection
    """

    def __init__(self, driver: Union[Firefox, Chrome], selector: str, selector_type: str = By.XPATH, timeout: int = 5,
                 is_collection_item: bool = False):

        self.selector_type = selector_type
        self.selector = selector
        self.driver = driver
        self.timeout = timeout
        self.is_collection_item = is_collection_item

    def _finder(self):
        """Ищет элемент на странице"""
        return self.driver.find_element(by=self.selector_type, value=self.selector)

    def __getattr__(self, item):
        return getattr(self._finder(), item)

    def exist(self) -> bool:
        """Возвращает существует ли элемент на странице"""
        try:
            self._finder()
        except NoSuchElementException:
            return False
        return True

    def get_index(self) -> Optional[int]:
        """Если элемент в коллекции возвращает его индекс иначе None"""
        if self.is_collection_item:
            return int(self.selector.split('[')[-1][:-1]) - 1
        else:
            return None

    def __str__(self):
        return "{0} [selector_type={1}, selector={2}, timeout={3}]".format(self.__class__.__name__,
                                                                           self.selector_type,
                                                                           self.selector,
                                                                           self.timeout)

    def assure(self, condition, *args, **kwargs):
        """Возвращает элемент если выполняется условие condition"""

        timeout = self.timeout if 'timeout' not in kwargs.keys() else kwargs['timeout']
        condition = condition(self, *args)
        try:
            WebDriverWait(self.driver, timeout).until(condition, str(self) + condition.error_msg_postfix)
        except TimeoutException:
            WebDriverWait(self.driver, 1).until(condition, str(self) + condition.error_msg_postfix)
        return self


class WaitingElement(BaseWaitingElement):
    """

    WaitingElement. Ищет элемент с указанным xpath.

    Attributes:
        driver: WebDriver
        selector: Селектор элементов
        selector_type: Тип селектора
        timeout: Время на поиск элемента
        is_collection_item: Является ли элементов коллекции. Используется при обьявлении WaitingElementCollection
    """

    def __init__(self, driver: Union[Firefox, Chrome], selector: str, selector_type: str = By.XPATH, timeout: int = 5,
                 is_collection_item: bool = False):
        super().__init__(driver, selector, selector_type, timeout, is_collection_item)

    def scroll_to_element(self):
        """
        Прокручивает страницу до элемента.
        :rtype: WaitingElement
        """
        location = self._finder().location
        self.driver.execute_script("window.scrollTo({0}, {1});".format(location['x'], location['y']))
        return self

    def mouse_over_element(self):
        """
        Наводит курсор мыши на элемент.
        :rtype: WaitingElement
        """
        hov = ActionChains(self.driver).move_to_element(self._finder())
        hov.perform()
        return self


class AndroidWaitingElement(BaseWaitingElement):
    """

    AndroidWaitingElement. Ищет элемент с указанным xpath.

    Attributes:
        driver: WebDriver
        selector: Селектор элементов
        selector_type: Тип селектора
        timeout: Время на поиск элемента
        is_collection_item: Является ли элементов коллекции. Используется при обьявлении WaitingElementCollection
    """

    def __init__(self, driver: Union[Firefox, Chrome], selector: str, selector_type: str = By.XPATH, timeout: int = 5,
                 is_collection_item: bool = False):
        super().__init__(driver, selector, selector_type, timeout, is_collection_item)


class SubWaitingElement(WaitingElement):
    """

    Класс для работы с субэлементами. Возвращает елемент внутри элемента waiting_element с добавлением к xpath первого
    sub_selector.

    Attributes:
        waiting_element: Родительский элемент.
        sub_selector: Дополнительных xpath.
        element_type: Тип элемента.
        is_collection_item: Является ли элементом коллекции. Используется при обьявлении WaitingElementCollection

    """
    def __init__(self, waiting_element, sub_selector, element_type=None, is_collection_item=False):
        super().__init__(waiting_element.driver,
                         waiting_element.selector + sub_selector,
                         waiting_element.selector_type,
                         waiting_element.timeout,
                         is_collection_item=is_collection_item)

        if element_type:
            self.__class__ = element_type
            element_type.__init__(self, waiting_element.driver,
                                  waiting_element.selector + sub_selector,
                                  waiting_element.selector_type,
                                  waiting_element.timeout,
                                  is_collection_item=is_collection_item)
